<?php
/**
 * @file transport.remote_entity.user.inc
 * Contains the remote entity class, interfaces, and hooks for users.
 */

/**
 * Entity class for users.
 *
 * The entity class should contain methods for phases that do not have interfaces;
 * ie those that are identical no matter the source.
 */
class TransportRemoteEntity_user extends TransportRemoteEntity {
  /**
   * The name of the property that holds the entity's id.
   */
  public $primary_key = 'uid';

  /**
   * Phase handler: user - init.
   */
  function run_phase_init() {
    // It is extremely important to call the parent method here.
    parent::run_phase_init();

    // Skip handling of users 0 and 1 from the remote site.
    if ($this->data['id'] == 0 || $this->data['id'] == 1) {
      $this->advance_phase = FALSE;
      $this->status = 'skip';
      //dsm($this, 'skip user');
    }
  }
  
  /**
   * Phase handler: user - dependencies.
   * 
   * Users don't have dependencies -- at least in core D6.
   * Well... that's not quite true. They have file dependencies, but uploading
   * user picture creates a TEMPORARY entry in the {files} table but only a
   * path is stored for the user -- WTF?
   * I really don't feel like accounting for this crusty part of core, so it
   * can stay unsupported until someone needs it ;)
   */  
  function run_phase_dependencies() {
    // TODO: Invoke the hook anyway as other modules may create dependencies.
  }
  
  /**
   * Phase handler: user - translate.
   */  
  function run_phase_translate() {
    // Nothing to do: no dependencies. 
    // TODO: Invoke the hook anyway as other modules may create dependencies.
  }
  
  /*
  !! TODO: need an extra phase: 'preexisting' --
  for users and files, (and terms?) check for an existing, identical entity.
  ie for user, check email
  files, filename + node
  etc
  */
  
  /**
   * Phase handler: user - save.
   */  
  function run_phase_save() {
    // user_save() is WEIRD.
    // TODO: clean up this code.
    
    $remote_user = $this->entity;
    //dsm($remote_user, 'rem user');

    // Note that the $remote_user's uid property has already been either
    // unset or set to the local value.
    if ($this->local_exists) {
      $local_user = user_load($this->id_local);

      $user_array = (array) $remote_user;
      $new_user = user_save($local_user, $user_array);
    }
    else {
      $user_array = (array) $remote_user;
      $new_user = user_save('', $user_array);
      
      $this->id_local = $new_user->uid;
      //dsm($new_user);

      // Password would have been doubly md5'ed, so set explicitly
      db_query("UPDATE {users} SET pass = '%s' WHERE uid = %d", $user_array['pass'], $new_user->uid);
    }
  }
}

/**
 * Extender Class: fetch phase.
 *
 * Type:   User.
 * Source: Clients Drupal 5
 */
class TransportRemoteEntityExtender_fetch_user_d5 
  extends TransportRemoteEntityExtender_fetch_client 
  implements TransportRemoteEntityInterface_fetch {
  
  /**
   * The name of the method for run_phase_fetch() to use.
   */
  public $fetch_method_name = 'user.load';
  
  /**
   * Phase handler: user - filter - D5.
   */
  function run_phase_filter() {
    // We don't handle roles yet, so clear them as this could be a
    // security issue.
    $this->object->entity->roles = array();
    
    // TODO: Clear/check 'theme' property.
    
    //dsm($this->object->entity);
  }  
}

/**
 * Extender Class: fetch phase.
 *
 * Type:   User.
 * Source: Clients Drupal 6 Services 2
 */
class TransportRemoteEntityExtender_fetch_user_d6_2
  extends TransportRemoteEntityExtender_fetch_client 
  implements TransportRemoteEntityInterface_fetch {
  
  /**
   * The name of the method for run_phase_fetch() to use.
   */
  public $fetch_method_name = 'user.get';
  
  /**
   * Phase handler: user - filter - D5.
   */
  function run_phase_filter() {
    // We don't handle roles yet, so clear them as this could be a
    // security issue.
    $this->object->entity->roles = array();
    
    // TODO: Clear/check 'theme' property.
    
    //dsm($this->object->entity);
  }  
}
