<?php
/**
 * @file transport.remote_entity.vocabulary.inc
 * Contains the remote entity class, interfaces, and hooks for terms.
 *
 * Requires taxonomy_get_service module on the remote site, part of the
 * taxonomy_services project.
 */

/**
 * Entity class for vocabularies.
 *
 * The entity class should contain methods for phases that do not have interfaces;
 * ie those that are identical no matter the source.
 */
class TransportRemoteEntity_vocabulary extends TransportRemoteEntity {
  /**
   * The name of the property that holds the entity's id.
   */
  public $primary_key = 'vid';

  /**
   * The name of the function to load a local entity.
   * A single parameter if its id is assumed by run_phase_local_load().
   */
  public $entity_load_function = 'taxonomy_vocabulary_load';
  // TODO: check this actually gets us everything we need.

  /**
   * Phase handler: vocabulary - init.
   */
  function run_phase_init() {
    // It is extremely important to call the parent method here.
    parent::run_phase_init();
    
    // Vocabularies rarely change...
    // Consider skipping a vocabulary when it's a dependency of a term, as
    // opposed to being explicitly requested.
    // We don't yet have the information to make that decision though -- 
    // see http://drupal.org/node/1093912.
    /*
    if ($this->local_exists) {
      $this->advance_phase = FALSE;
      $this->status = 'skip';
      //dsm($this, 'skip vocabulary');
    }
    */
  }

  /**
   * Phase handler: vocabulary - save.
   */  
  function run_phase_save() {
    //dsm($this->entity, 'about to save vocabulary');
    
    // taxonomy_save_vocabulary() is not a proper API function but a form
    // helper... oh wait a minute haven't I said that already? *sigh*
    $vocabulary_array = (array) $this->entity;
    
    taxonomy_save_vocabulary($vocabulary_array);
    
    // Because we didn't save our actual entity object, we have to retrieve
    // the (potentially) new vid manually.
    $this->entity->vid = $vocabulary_array['vid'];

    // Set the local ID on the remote entity object.
    $this->id_local = $this->entity->vid;
  }
}

/**
 * Extender Class: fetch phase.
 *
 * Type:   Vocabulary.
 * Source: Clients Drupal 5
 */
class TransportRemoteEntityExtender_fetch_vocabulary_d5
  extends TransportRemoteEntityExtender_fetch_client 
  implements TransportRemoteEntityInterface_fetch {
    
  /**
   * The name of the method for run_phase_fetch() to use.
   */
  public $fetch_method_name = 'taxonomy.loadVocabulary';
  
  /**
   * Phase handler: vocabulary - filter.
   *
   * The filter phase performs any clean-up required according to the object's version.
   */
  function run_phase_filter() {    
    $remote_vocabulary = $this->object->entity;

    // The format of the vocabulary's nodes array changed from D5 to D6, from
    // flat to keyed by type.
    $nodes = $remote_vocabulary->nodes;
    $remote_vocabulary->nodes = array();
    foreach ($nodes as $node_type) {
      $remote_vocabulary->nodes[$node_type] = $node_type;
    }
  }
}

/**
 * Extender Class: fetch phase.
 *
 * Type:   Vocabulary.
 * Source: Clients Drupal 6 Services 2
 */
class TransportRemoteEntityExtender_fetch_vocabulary_d6_2 
  extends TransportRemoteEntityExtender_fetch_client 
  implements TransportRemoteEntityInterface_fetch {
    
  /**
   * The name of the method for run_phase_fetch() to use.
   */
  public $fetch_method_name = 'taxonomy.loadVocabulary';
  
  /**
   * Phase handler: vocabulary - filter.
   *
   * The filter phase performs any clean-up required according to the object's version.
   */
  function run_phase_filter() {    
  }
}
